/*
 * (C) Copyright 2007 Catalin Patulea <cat@vv.carleton.ca>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */
#ifndef FIRMWARE_H
#define FIRMWARE_H

typedef unsigned int u32 ;
typedef unsigned char u8 ;
typedef unsigned short u16 ;
typedef unsigned long size_t;
typedef void FILE;

/* Firmware system call entry points (thanks shirour) */
#define FIRMWARE_FOPEN	0x0014CB64
#define FIRMWARE_FREAD	0x0014CC7C
#define FIRMWARE_FCLOSE	0x0014CBD0

/* Thin wrappers around system calls */
static inline FILE *fopen(const char *path, const char *mode) {
	typedef FILE *(*fopen_t)(const char *, const char *);
	return ((fopen_t)FIRMWARE_FOPEN)(path, mode);
}

static inline size_t fread(void *ptr, size_t size, size_t nmemb, FILE *stream) {
	typedef size_t (*fread_t)(void *, size_t, size_t, FILE *);
	return ((fread_t)FIRMWARE_FREAD)(ptr, size, nmemb, stream);
}

static inline int fclose(FILE *fp) {
	typedef int (*fclose_t)(FILE *);
	return ((fclose_t)FIRMWARE_FCLOSE)(fp);
}

#endif
