/*
 * (C) Copyright 2007 Catalin Patulea <cat@vv.carleton.ca>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */
#include "dm320.h"
#include "uart.h"
#include "firmware.h"

#define CONFIG_KERNEL_FILENAME      "a:/rockbox.mrboot"
#define CONFIG_KERNEL_LOAD_ADDR     0x01900000

static inline void setup_cpu(void) {
	/* Disable all interrupts, disable/flush caches, change to SVC mode */
	uartPuts("Initializing CPU... ");

	/* Set up interrupt controller (INTC) */
	IO_INTC_ENTRY_TBA0=0;
	IO_INTC_ENTRY_TBA1=0;

	/* Okay, I'll fess up. I don't know what half this crap does. But it seems
	 * to work. Someone please document this.
	 */
	__asm__ __volatile__(
		/* Disable data cache */
		"mrc	p15, 0, r0, c1, c0, 0\n"
		"bic	r0, r0, #0x00002300\n"	/* clear bits 13, 9:8 (--V- --RS) */
		"bic	r0, r0, #0x00000087\n"	/* clear bits 7, 2:0 (B--- -CAM) */
		"orr	r0, r0, #0x00000002\n"	/* set bit 2 (A) Align */
		"orr	r0, r0, #0x00001000\n"	/* set bit 12 (I) I-Cache */
		"mcr	p15, 0, r0, c1, c0, 0\n"
		
		/* Flush data cache and TLB */
		"mov	r0,#0\n"
		"1: mrc	p15, 0, r15, c7, c10, 3\n"
		"bne	1b\n"
		
		"mov	r0,#0\n"
		"mcr	p15, 0, r0, c7, c10, 4\n"

		"mov	r0,#0\n"
		"mcr	p15, 0, r0, c7, c5, 0\n"

		"mov	r0,#0\n"
		"mcr	p15, 0, r0, c7, c6, 0\n"
		"mcr	p15, 0, r0, c8, c7, 0\n"	/* flush v4 TLB */

		/* SVC mode with IRQs disabled */
		"mrs	r0,cpsr\n"
		"bic	r0,r0,#0x1f\n"
		"orr	r0,r0,#0xd3\n"
		"msr	cpsr,r0"
		:
		:
		: "memory", "r0");
	
	uartPuts("done.\r\n");
}

static inline size_t setup_image(const char *filename, char *base, unsigned int failok) {
	FILE *fp;
	size_t read, total;
	register int checksum = 0;
	
	uartPuts("Opening ");
	uartPuts(filename);
	uartPuts("... ");
	
	/* Open the kernel image file */
	fp = fopen(filename, "rb");
	if (fp == 0) {
		uartPuts("Image missing, ");
		if (failok) {
			uartPuts("continuing.\r\n");
			return 0;
		} else {
			uartPuts("HALT.\r\n");
			for (;;);
		}
	}
	
	uartPuts("done.\r\nLoading image");
	
	/* Read it all, 256 KB at a time */
	total = 0;
	while ((read = fread(base, 1, 0x40000, fp))) {
		total += read;
		while (read--) {
			checksum = (checksum >> 1) + ((checksum & 1) << 15);
			checksum += *base++;
			checksum &= 0xffff;
		}
		
		uartPuts(".");
	}
	
	/* Close the file */
	fclose(fp);
	
	uartPuts(" 0x");
	uartPutHex(total);
	uartPuts(" bytes done, checksum 0x");
	uartPutHex(checksum);
	uartPuts("\r\n");
	
	return total;
}

void main(void) __attribute__((naked));
void main(void) {
    void(*kernel_entry)(void);

	/* Mask timer interrupts so firmware doesn't send commands for the remote
	 * (leave HDD and most other interrupts unmasked or file system calls
	 * become very slow)
	 */
	IO_INTC_EINT0 &= ~15;

	/* Print a banner */
	uartSetup();
	uartPuts("\r\n\r\nm:robe 500i loader v1.0 (" __DATE__ ")\r\n");
	
	/* Load kernel image (zImage) */
	setup_image(CONFIG_KERNEL_FILENAME, (char *)CONFIG_KERNEL_LOAD_ADDR, 0);

	setup_cpu();

    kernel_entry = (void (*) ())CONFIG_KERNEL_LOAD_ADDR;
    kernel_entry();

	for (;;);
}
